@@ -27,7 +27,7 @@ use constant DATASET_HASH_LIMIT => 5001;
 		# region, and a hash returned with potentially a score for each basepair. 
 		# This may become unwieldy for very large regions, which may be better 
 		# served by separate database queries for each window.
-my $VERSION = '1.34';
+my $VERSION = '1.40';
 
 print "\n This script will collect binned values across features\n\n";
 
@@ -124,7 +124,7 @@ my $start_time = time;
 ### Generate or load the dataset
 my $Data;
 if ($infile) {
-	$Data = Bio::ToolBox::Data->new(file => $infile) or 
+	$Data = Bio::ToolBox::Data->new(file => $infile, parse => 1) or 
 		die " unable to load input file '$infile'\n";
 	printf " Loaded %s features from $infile.\n", format_with_commas( $Data->last_row );
 	
@@ -498,7 +498,7 @@ sub collect_binned_data_for_features {
 		my $feature = $row->feature;
 		unless ($feature) {
 			# record null values
-			for my $c ($startcolumn..($Data->number_columns - 1) ) {
+			for my $c ($startcolumn..($Data->last_column) ) {
 				$row->value($c, '.');
 			}
 			next;
@@ -513,7 +513,7 @@ sub collect_binned_data_for_features {
 			# we will skip this feature
 			
 			# but first, put in null values
-			for my $c ($startcolumn..($Data->number_columns - 1) ) {
+			for my $c ($startcolumn..($Data->last_column) ) {
 				$row->value($c, '.');
 			}
 			next;
@@ -582,7 +582,7 @@ sub collect_binned_data_for_regions {
 			# we will skip this feature
 			
 			# but first, put in null values
-			for my $c ($startcolumn..($Data->number_columns - 1) ) {
+			for my $c ($startcolumn..($Data->last_column) ) {
 				$row->value($c, '.');
 			}
 			next;
@@ -638,7 +638,7 @@ sub record_the_bin_values {
 	
 	
 	# assign the scores to the bins in the region
-	for my $column ($startcolumn..($Data->number_columns - 1) ) {
+	for my $column ($startcolumn..($Data->last_column) ) {
 		# we will step through each data column, representing each window (bin)
 		# across the feature's region
 		# any scores within this window will be collected and the mean 
@@ -773,7 +773,7 @@ sub record_individual_bin_values {
 	my ($row, $chromo, $fstart, $fstop, $strand, $length) = @_;
 	
 	# collect the scores to the bins in the region
-	for my $column ($startcolumn..($Data->number_columns - 1) ) {
+	for my $column ($startcolumn..($Data->last_column) ) {
 		# we will step through each data column, representing each window (bin)
 		# across the feature's region
 		# any scores within this window will be collected and the mean 
@@ -863,7 +863,7 @@ sub record_individual_bin_values {
 sub go_interpolate_values {
 	
 	# determine counts
-	my $lastwindow = $Data->number_columns - 1; 
+	my $lastwindow = $Data->last_column; 
 		# lastwindow is the index of the last column
 	
 	# walk through each data line and then each window
@@ -1020,7 +1020,7 @@ A program to collect data in bins across a list of features.
  get_binned_data.pl [--options] <filename>
   
   Options for existing files:
-  --in <filename>
+  --in <filename>                  (txt bed gff gtf refFlat ucsc)
   
   Options for new files:
   --db <name | filename>
@@ -1051,7 +1051,7 @@ A program to collect data in bins across a list of features.
   --gz
   --cpu <integer>                                           (2)
   --version
-  --help
+  --help                              show extended documentation
 
 =head1 OPTIONS
 
@@ -1062,12 +1062,12 @@ The command line flags and descriptions:
 =item --in <filename>
 
 Specify an input file containing either a list of database features or 
-genomic coordinates for which to collect data. The file should be a 
-tab-delimited text file, one row per feature, with columns representing 
-feature identifiers, attributes, coordinates, and/or data values. The 
-first row should be column headers. Bed files are acceptable, as are 
-text files generated by other B<BioToolBox> scripts. Files may be 
-gzipped compressed.
+genomic coordinates for which to collect data. Any tab-delimited text 
+file with recognizable headers is supported. Gene annotation file 
+formats are also supported, including bed, gtf, gff3, refFlat, and 
+UCSC native formats such as gene prediction tables are all supported. 
+Gene annotation files will be parsed as sequence features. 
+Files may be gzipped compressed.
 
 =item --db <name | filename>
 
